#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    import usb.core
except:
    raise Exception("You need pyusb-1 to run this script!")

import array, time


class SimpleLogger:
    def __init__(self, rawFileName):
        self.rawFile = open(rawFileName, "a")
        self.sensorFiles = {}

    def log(self, data):
        print "\t".join([str(i) for i in data])
        s = data[1]
        if not s in self.sensorFiles:
            self.sensorFiles[s] = open(s+".log", "a")
        self.sensorFiles[s].write("%s\t%s\t%s\t%s\n" % \
            (time.time(), data[3], data[0], data[2]))
        self.sensorFiles[s].flush()

    def logRaw(self, rawData):
        self.rawFile.write("%15f" % time.time())
        for d in rawData:
            self.rawFile.write(" %02X" % d)
        self.rawFile.write("\n")
        self.rawFile.flush()


class TL500:
    def __init__(self, sensors={}, action=None, rawAction=None):
        self.sensors = sensors
        self.dev = None
        self.action = action
        self.rawAction = rawAction
        self._buffer = array.array('B', [0]*64)
        
    def connect(self):
        self.dev = usb.core.find(idVendor=0x0451, idProduct=0x3211)
        if not self.dev:
            raise Exception("Arexx TL-500 not found!")

    def loopCollect(self):
        if not self.dev:
            self.connect()
        self._buffer[0] = 4
        self.dev.write(0x1, self._buffer, 0, 1000)
        self._buffer[0] = 3
        while True:
            self.dev.write(0x1, self._buffer, 0, 1000)
            rawData = self.dev.read(0x81, 64, 0, 1000)
            if rawData[1] != 0:
                if self.rawAction:
                    self.rawAction(rawData)
                if self.action:
                    self.action(self.processData(rawData))
            time.sleep(10)

    def processData(self, rawData):
        sensor   = rawData[3]*256 + rawData[2]
        valueRaw = rawData[4]*256 + rawData[5]
        clock    = rawData[9]*16777216 + rawData[8]*65536 + \
            rawData[7]*256 + rawData[6]
        # FIXME: this should changed for other sensor types
        value    = 0.0078 * valueRaw 
        return clock, self.sensors.get(sensor, sensor), valueRaw, value


if __name__ == "__main__":
    logger = SimpleLogger("raw.txt")
    # these are the names of my sensors
    sensors = {11492: "325_in", 11772: "325_out", \
        8857: "325_srv", 8480: "323_in"}
    tl500 = TL500(sensors=sensors, action=logger.log, rawAction=logger.logRaw)
    errors = 0
    while errors <= 1000:
        try:
            tl500.loopCollect()
        except Exception, e:
            print e
            errors += 1
